<?php

namespace Drupal\cforge\Forms;

use Drupal\mcapi\Entity\Transaction;
use Drupal\user\Entity\User;
use Drupal\mcapi\Storage\WalletStorage;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Cache\Cache;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder to choose who referred you to the site.
 */
class ChooseReferrer extends FormBase {

  protected $logger;

  public function __construct(LoggerChannel $logger_channel) {
    $this->logger = $logger_channel;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.channel.cforge')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hamlets_referrer_form';
  }

  /**
   * {@inheritdoc}
   *
   * @todo Exclude the current user from the selection handler.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $user = NULL) {
    $form['referrer'] = [
      '#title' => $this->t('Who encouraged you to join this site?'),
      '#description' => $this->t('Tell us and they will receive a small reward.'),
      '#type' => 'entity_autocomplete',
      '#target_type' => 'user',
      '#selection_handler' => 'alt_login:user',
      '#selection_settings' => [
        'include_anonymous' => 0,
        'filter' => [
          'type' => 'role',
          'role' => [RID_TRADER => RID_TRADER],
        ],
        'sort' => [
          'field' => 'name',
          'direction' => 'ASC',
        ],
      ],
      '#placeholder' => t('Name...'),
      '#tags' => FALSE,
    ];
    $form['new_user'] = [
      '#type' => 'value',
      '#value' => $user,
    ];
    $form['actions'] = [
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Pay'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $referrer_uid = $form_state->getValue('referrer');

    $new_user_uid = $form_state->getValue('new_user');
    if (!$referrer_uid) {
      return;
    }
    if ($referrer_uid == $new_user_uid) {
      $form_state->setErrorByName('new_user', $this->t("You cannot refer yourself."));
      return;
    }
    $referrer = User::load($referrer_uid);
    $settings = \Drupal::config('cforge.settings');
    $props = [
      'payer' => $settings->get('referral_src'),
      'payee' => WalletStorage::firstWalletOfEntity($referrer),
      'type' => 'auto',
      'worth' => $settings->get('referral_fee'),
      'description' => $this->t('Reward for referring new member @mem', ['@mem' => User::load($new_user_uid)->label()]),
      'category' => $settings->get('referral_cat')
    ];
    $transaction = Transaction::create($props);
    $violations = $transaction->validate();
    if ($violations->count()) {
      $this->logger->error(
        'Failed to pay referrer: @transaction',
        ['@transaction' => '<pre>'.print_r($transaction->toArray(), 1).'</pre>']
      );
      $form_state->setErrorByName(
        'referrer',
        $this->t('Sorry, the transaction failed. Errors have been logged.')
      );
      foreach ($violations as $violation) {
         $this->logger->error($violation->getMessageTemplate(), $violation->getParameters());
      }
    }
    else {
      $form_state->set('mcapi_transaction', $transaction);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->get('mcapi_transaction')->save();
    $uid = $form_state->getValue('new_user');
    // @this should work but I'm  not sure if this is the best way to clear the block access cache
    $payee_user = $form_state->get('mcapi_transaction')->payee->entity->getHolder();
    \Drupal::keyValue('user_referrer')->set($uid, $payee_user->id());

    Cache::invalidateTags(['user:' . $uid, 'user:' . $payee_user->id()]);
    $this->logger->info(
      'Referrer reward given to @name for referring @new_name',
      [
        '@name' => $payee_user->getDisplayName(),
        '@new_name' => User::load($form_state->getValue('new_user'))->getDisplayName(),
      ]
    );
    \Drupal::messenger()->addStatus(
      $this->t('@name has been given @amount for referring you!'),
      [
        '@name' => $payee_user->toLink(),
        '@amount' => $form_state->get('mcapi_transaction')->worth->__toString(),
      ]
    );
  }

}
